@@ -11,14 +11,21 @@ Module includes functions for easy access to YahooFinance data
 """
 
 
-from datetime import datetime, date
+
 import urllib.request
-from pandas import DataFrame, Index, HDFStore, Panel
 import numpy as np
-import os
+
+import requests # interaction with the web
+import os  #  file system operations
+import yaml # human-friendly data format
+import re  # regular expressions
+import pandas as pd # pandas... the best time series library out there
+import datetime as dt # date and time functions
+import io 
+
 from .extra import ProgressBar
 
-
+dateTimeFormat = "%Y%m%d %H:%M:%S"
 
 def parseStr(s):
     ''' convert string to a float or string '''
@@ -40,76 +47,7 @@ def parseStr(s):
         except ValueError: # failed, return original string
             return s
 
-class HistData(object):
-    ''' a class for working with yahoo finance data '''
-    def __init__(self, autoAdjust=True):
-       
-        self.startDate = (2008,1,1)
-        self.autoAdjust=autoAdjust
-        self.wp = Panel()
-        
-        
-    def load(self,dataFile):
-        """load data from HDF"""
-        if os.path.exists(dataFile):
-            store = HDFStore(dataFile)
-            symbols = [str(s).strip('/') for s in list(store.keys()) ]   
-            data = dict(list(zip(symbols,[store[symbol] for symbol in symbols])))
-            self.wp = Panel(data)
-            store.close()
-        else:
-            raise IOError('Data file does not exist')
-            
-        
-    def save(self,dataFile):
-        """ save data to HDF"""
-        print(('Saving data to', dataFile))
-        store = HDFStore(dataFile)
-        for symbol in self.wp.items:
-            store[symbol] = self.wp[symbol]
-            
-        store.close()
-                    
-            
-            
-    def downloadData(self,symbols='all'):
-        ''' get data from yahoo  '''
-        
-        if symbols == 'all':
-            symbols = self.symbols
-        
-        #store = HDFStore(self.dataFile)        
-        p = ProgressBar(len(symbols))
-        
-        for idx,symbol in enumerate(symbols):
-            
-            try:            
-                df = getSymbolData(symbol,sDate=self.startDate,verbose=False)
-                if self.autoAdjust:
-                    df =  _adjust(df,removeOrig=True)
-                
-                if len(self.symbols)==0:
-                    self.wp = Panel({symbol:df})
-                else:
-                    self.wp[symbol] = df
-            
-            except Exception as e:
-                print(e) 
-            p.animate(idx+1)
-    
-    def getDataFrame(self,field='close'):
-        ''' return a slice on wide panel for a given field '''
-        return self.wp.minor_xs(field)
-         
-    
-    @property
-    def symbols(self):
-        return self.wp.items.tolist()        
-           
-  
-    def __repr__(self):
-        return str(self.wp)
-
+   
 
 def getQuote(symbols):
     """ 
@@ -155,21 +93,8 @@ def getQuote(symbols):
     idx = data.pop('symbol')
     
     
-    return DataFrame(data,index=idx)
+    return pd.DataFrame(data,index=idx)
 
-def _historicDataUrll(symbol, sDate=(1990,1,1),eDate=date.today().timetuple()[0:3]):
-    """ 
-    generate url
-
-    symbol: Yahoo finanance symbol
-    sDate: start date (y,m,d)
-    eDate: end date (y,m,d)
-    """
-
-    urlStr = 'http://ichart.finance.yahoo.com/table.csv?s={0}&a={1}&b={2}&c={3}&d={4}&e={5}&f={6}'.\
-    format(symbol.upper(),sDate[1]-1,sDate[2],sDate[0],eDate[1]-1,eDate[2],eDate[0])
-    
-    return urlStr
 
 def getHistoricData(symbols, **options):
     ''' 
@@ -183,8 +108,6 @@ def getHistoricData(symbols, **options):
         Yahoo finanance symbol or a list of symbols
     sDate : tuple  (optional)
         start date (y,m,d)
-    eDate : tuple  (optional)
-        end date (y,m,d) 
     adjust : bool
         T/[F] adjust data based on adj_close
     
@@ -206,9 +129,9 @@ def getHistoricData(symbols, **options):
             p.animate(idx+1)
             data[symbol] = getSymbolData(symbol,verbose=False,**options)
         
-        return Panel(data)
+        return pd.Panel(data)
 
-def getSymbolData(symbol, sDate=(1990,1,1), eDate=None, adjust=False, verbose=True):
+def getSymbolData(symbol, sDate=(2000,1,1), adjust=False, verbose=True):
     """ 
     get data from Yahoo finance and return pandas dataframe
 
@@ -218,8 +141,6 @@ def getSymbolData(symbol, sDate=(1990,1,1), eDate=None, adjust=False, verbose=Tr
         Yahoo finanance symbol
     sDate : tuple , optional
         start date (y,m,d), defaults to 1 jan 1990
-    eDate : tuple , optional
-        end date (y,m,d), defaults to current date
     adjust : bool , optional
         use adjusted close values to correct OHLC. adj_close will be ommited
     verbose : bool , optional
@@ -230,38 +151,23 @@ def getSymbolData(symbol, sDate=(1990,1,1), eDate=None, adjust=False, verbose=Tr
         DataFrame
             
     """
-
     
-    if eDate is None: eDate = date.today().timetuple()[0:3]
+    period1 = int(dt.datetime(*sDate).timestamp()) # convert to seconds since epoch
+    period2 = int(dt.datetime.now().timestamp())
     
-
-    urlStr = 'http://ichart.finance.yahoo.com/table.csv?s={0}&a={1}&b={2}&c={3}&d={4}&e={5}&f={6}'.\
-    format(symbol.upper(),sDate[1]-1,sDate[2],sDate[0],eDate[1]-1,eDate[2],eDate[0])
-
+    params = (symbol, period1, period2, _token['crumb'])
     
-    try:
-        lines = urllib.request.urlopen(urlStr).readlines()
-    except Exception as e:
-        s = "Failed to download:\n{0}".format(e);
-        print(s)
-        return None
+    url = "https://query1.finance.yahoo.com/v7/finance/download/{0}?period1={1}&period2={2}&interval=1d&events=history&crumb={3}".format(*params)
 
-    dates = []
-    data = [[] for i in range(6)]
-    #high
-    # header : Date,Open,High,Low,Close,Volume,Adj Close
-    for line in lines[1:]:
-        #print line
-        fields = line.decode().rstrip().split(',')
-        dates.append(datetime.strptime( fields[0],'%Y-%m-%d'))
-        for i,field in enumerate(fields[1:]):
-            data[i].append(float(field))
-       
-    idx = Index(dates)
-    data = dict(list(zip(['open','high','low','close','volume','adj_close'],data)))
+    data = requests.get(url, cookies={'B':_token['cookie']})
     
-    # create a pandas dataframe structure   
-    df = DataFrame(data,index=idx).sort_index()
+    buf = io.StringIO(data.text) # create a buffer
+    df = pd.read_csv(buf,index_col=0,parse_dates=True) # convert to pandas DataFrame
+    
+    # rename columns
+    newNames = [c.lower().replace(' ','_') for c in df.columns]    
+    renames = dict(zip(df.columns,newNames))    
+    df = df.rename(columns=renames)
     
     if verbose:
         print(('Got %i days of data' % len(df)))
@@ -288,17 +194,89 @@ def _adjust(df, removeOrig=False):
     
     return df
     
-def getScreenerSymbols(fileName):
-    ''' read symbols from a .csv saved by yahoo stock screener '''
+
+def loadToken():
+    """ 
+    get cookie and crumb from APPL page or disk. 
+    force = overwrite disk data
+    """
+    refreshDays = 30 # refreh cookie every x days
     
-    with open(fileName,'r') as fid:
-        lines = fid.readlines()
+    # set destinatioin file
+    dataDir = os.path.expanduser('~')+'/twpData'
+    dataFile = dataFile = os.path.join(dataDir,'yahoo_cookie.yml')
+    
+    try : # load file from disk
+        
+        data = yaml.load(open(dataFile,'r'))
+        age = (dt.datetime.now()- dt.datetime.strptime(  data['timestamp'], dateTimeFormat) ).days
+        assert age < refreshDays, 'cookie too old'
+            
+    except (AssertionError,FileNotFoundError):     # file not found
+        
+        if not os.path.exists(dataDir):
+            os.mkdir(dataDir)
+        
+        data = getToken(dataFile)
+        
+    return data
 
-    symbols = []   
-    for line in lines[3:]:
-        fields = line.strip().split(',')
-        field = fields[0].strip()
-        if len(field) > 0:
-            symbols.append(field) 
-    return symbols
 
+def getToken(fName = None):
+    """ get cookie and crumb from yahoo """
+    
+    url = 'https://uk.finance.yahoo.com/quote/AAPL/history' # url for a ticker symbol, with a download link
+    r = requests.get(url)  # download page
+    
+    txt = r.text # extract html
+    
+    cookie = r.cookies['B'] # the cooke we're looking for is named 'B'
+    
+    pattern = re.compile('.*"CrumbStore":\{"crumb":"(?P<crumb>[^"]+)"\}')
+    
+    for line in txt.splitlines():
+        m = pattern.match(line)
+        if m is not None:
+            crumb = m.groupdict()['crumb']   
+    
+    assert r.status_code == 200 # check for succesful download
+            
+    # save to disk
+    data = {'crumb': crumb, 'cookie':cookie, 'timestamp':dt.datetime.now().strftime(dateTimeFormat)}
+
+    if fName  is not None: # save to file
+        with open(fName,'w') as fid:
+            yaml.dump(data,fid)
+    
+    return data
+
+
+#-------------- get token
+_token = loadToken() # get token from disk or yahoo
+
+#--------------tests------------
+# to be executed with pytest
+
+
+def test_getToken():
+    ''' download token '''
+    getToken()   
+    
+    
+def test_initToken():
+    ''' remove and get token '''
+    dataDir = os.path.expanduser('~')+'/twpData'
+    dataFile = dataFile = os.path.join(dataDir,'yahoo_cookie.yml')
+    
+    if os.path.exists(dataFile):
+        os.remove(dataFile)
+    
+    loadToken()
+    
+    assert os.path.exists(dataFile)
+    
+def test_download():
+    
+    vxx = getSymbolData('SPY')
+    
+    assert len(vxx) > 4000
\ No newline at end of file
